@@ -74,7 +74,6 @@ use core::marker::PhantomData;
 use core::mem;
 use core::num::NonZeroUsize;
 use core::ptr::{self, NonNull};
-use core::usize;
 
 #[cfg(any(test, not(feature = "hashbrown")))]
 extern crate std;
@@ -180,7 +179,7 @@ impl<K, V> LruEntry<K, V> {
 }
 
 #[cfg(feature = "hashbrown")]
-pub type DefaultHasher = hashbrown::hash_map::DefaultHashBuilder;
+pub type DefaultHasher = hashbrown::DefaultHashBuilder;
 #[cfg(not(feature = "hashbrown"))]
 pub type DefaultHasher = std::collections::hash_map::RandomState;
 
@@ -234,7 +233,7 @@ impl<K: Hash + Eq, V> LruCache<K, V> {
     /// let mut cache: LruCache<isize, &str> = LruCache::unbounded();
     /// ```
     pub fn unbounded() -> LruCache<K, V> {
-        LruCache::construct(NonZeroUsize::new(usize::MAX).unwrap(), HashMap::default())
+        LruCache::construct(NonZeroUsize::MAX, HashMap::default())
     }
 }
 
@@ -270,10 +269,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// let mut cache: LruCache<isize, &str> = LruCache::unbounded_with_hasher(s);
     /// ```
     pub fn unbounded_with_hasher(hash_builder: S) -> LruCache<K, V, S> {
-        LruCache::construct(
-            NonZeroUsize::new(usize::MAX).unwrap(),
-            HashMap::with_hasher(hash_builder),
-        )
+        LruCache::construct(NonZeroUsize::MAX, HashMap::with_hasher(hash_builder))
     }
 
     /// Creates a new LRU Cache with the given capacity.
@@ -522,6 +518,50 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a key-value references pair of the key in the cache or `None` if it is not
+    /// present in the cache. The reference to the value of the key is mutable. Moves the key to
+    /// the head of the LRU list if it exists.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    /// let (k, v) = cache.get_key_value_mut(&1).unwrap();
+    /// assert_eq!(k, &1);
+    /// assert_eq!(v, &mut "a");
+    /// *v = "aa";
+    /// cache.put(3, "c");
+    /// assert_eq!(cache.get_key_value(&2), None);
+    /// assert_eq!(cache.get_key_value(&1), Some((&1, &"aa")));
+    /// assert_eq!(cache.get_key_value(&3), Some((&3, &"c")));
+    /// ```
+    pub fn get_key_value_mut<'a, Q>(&'a mut self, k: &Q) -> Option<(&'a K, &'a mut V)>
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            Some(unsafe {
+                (
+                    &*(*node_ptr).key.as_ptr(),
+                    &mut *(*node_ptr).val.as_mut_ptr(),
+                )
+            })
+        } else {
+            None
+        }
+    }
+
     /// Returns a reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
     /// If the key does not exist the provided `FnOnce` is used to populate
@@ -544,7 +584,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// assert_eq!(cache.get_or_insert(1, ||"a"), &"a");
     /// assert_eq!(cache.get_or_insert(1, ||"b"), &"a");
     /// ```
-    pub fn get_or_insert<'a, F>(&'a mut self, k: K, f: F) -> &'a V
+    pub fn get_or_insert<F>(&mut self, k: K, f: F) -> &V
     where
         F: FnOnce() -> V,
     {
@@ -568,6 +608,57 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a reference is returned. The value referenced by the
+    /// key is only cloned (using `to_owned()`) if it doesn't exist in the
+    /// cache.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// use std::rc::Rc;
+    ///
+    /// let key1 = Rc::new("1".to_owned());
+    /// let key2 = Rc::new("2".to_owned());
+    /// let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+    /// assert_eq!(cache.get_or_insert_ref(&key1, ||"One".to_owned()), "One");
+    /// assert_eq!(cache.get_or_insert_ref(&key2, ||"Two".to_owned()), "Two");
+    /// assert_eq!(cache.get_or_insert_ref(&key2, ||"Not two".to_owned()), "Two");
+    /// assert_eq!(cache.get_or_insert_ref(&key2, ||"Again not two".to_owned()), "Two");
+    /// assert_eq!(Rc::strong_count(&key1), 2);
+    /// assert_eq!(Rc::strong_count(&key2), 2); // key2 was only cloned once even though we
+    ///                                         // queried it 3 times
+    /// ```
+    pub fn get_or_insert_ref<'a, Q, F>(&'a mut self, k: &'_ Q, f: F) -> &'a V
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized + alloc::borrow::ToOwned<Owned = K>,
+        F: FnOnce() -> V,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { &*(*node_ptr).val.as_ptr() }
+        } else {
+            let v = f();
+            let (_, node) = self.replace_or_create_node(k.to_owned(), v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            unsafe { &*(*node_ptr).val.as_ptr() }
+        }
+    }
+
     /// Returns a reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
     /// If the key does not exist the provided `FnOnce` is used to populate
@@ -619,6 +710,61 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a reference is returned. If `FnOnce` returns `Err`,
+    /// returns the `Err`. The value referenced by the key is only cloned
+    /// (using `to_owned()`) if it doesn't exist in the cache and `FnOnce`
+    /// succeeds.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// use std::rc::Rc;
+    ///
+    /// let key1 = Rc::new("1".to_owned());
+    /// let key2 = Rc::new("2".to_owned());
+    /// let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+    /// let f = ||->Result<String, ()> {Err(())};
+    /// let a = ||->Result<String, ()> {Ok("One".to_owned())};
+    /// let b = ||->Result<String, ()> {Ok("Two".to_owned())};
+    /// assert_eq!(cache.try_get_or_insert_ref(&key1, a), Ok(&"One".to_owned()));
+    /// assert_eq!(cache.try_get_or_insert_ref(&key2, f), Err(()));
+    /// assert_eq!(cache.try_get_or_insert_ref(&key2, b), Ok(&"Two".to_owned()));
+    /// assert_eq!(cache.try_get_or_insert_ref(&key2, a), Ok(&"Two".to_owned()));
+    /// assert_eq!(Rc::strong_count(&key1), 2);
+    /// assert_eq!(Rc::strong_count(&key2), 2); // key2 was only cloned once even though we
+    ///                                         // queried it 3 times
+    /// ```
+    pub fn try_get_or_insert_ref<'a, Q, F, E>(&'a mut self, k: &'_ Q, f: F) -> Result<&'a V, E>
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized + alloc::borrow::ToOwned<Owned = K>,
+        F: FnOnce() -> Result<V, E>,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { Ok(&*(*node_ptr).val.as_ptr()) }
+        } else {
+            let v = f()?;
+            let (_, node) = self.replace_or_create_node(k.to_owned(), v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            Ok(unsafe { &*(*node_ptr).val.as_ptr() })
+        }
+    }
+
     /// Returns a mutable reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
     /// If the key does not exist the provided `FnOnce` is used to populate
@@ -641,7 +787,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// assert_eq!(cache.get_or_insert_mut(3, ||"f"), &mut "f");
     /// assert_eq!(cache.get_or_insert_mut(3, ||"e"), &mut "f");
     /// ```
-    pub fn get_or_insert_mut<'a, F>(&'a mut self, k: K, f: F) -> &'a mut V
+    pub fn get_or_insert_mut<F>(&mut self, k: K, f: F) -> &mut V
     where
         F: FnOnce() -> V,
     {
@@ -665,6 +811,56 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a mutable reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a mutable reference is returned. The value referenced by the
+    /// key is only cloned (using `to_owned()`) if it doesn't exist in the cache.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// use std::rc::Rc;
+    ///
+    /// let key1 = Rc::new("1".to_owned());
+    /// let key2 = Rc::new("2".to_owned());
+    /// let mut cache = LruCache::<Rc<String>, &'static str>::new(NonZeroUsize::new(2).unwrap());
+    /// cache.get_or_insert_mut_ref(&key1, ||"One");
+    /// let v = cache.get_or_insert_mut_ref(&key2, ||"Two");
+    /// *v = "New two";
+    /// assert_eq!(cache.get_or_insert_mut_ref(&key2, ||"Two"), &mut "New two");
+    /// assert_eq!(Rc::strong_count(&key1), 2);
+    /// assert_eq!(Rc::strong_count(&key2), 2); // key2 was only cloned once even though we
+    ///                                         // queried it 2 times
+    /// ```
+    pub fn get_or_insert_mut_ref<'a, Q, F>(&mut self, k: &'_ Q, f: F) -> &'a mut V
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized + alloc::borrow::ToOwned<Owned = K>,
+        F: FnOnce() -> V,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { &mut *(*node_ptr).val.as_mut_ptr() }
+        } else {
+            let v = f();
+            let (_, node) = self.replace_or_create_node(k.to_owned(), v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            unsafe { &mut *(*node_ptr).val.as_mut_ptr() }
+        }
+    }
+
     /// Returns a mutable reference to the value of the key in the cache if it is
     /// present in the cache and moves the key to the head of the LRU list.
     /// If the key does not exist the provided `FnOnce` is used to populate
@@ -693,7 +889,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// assert_eq!(cache.try_get_or_insert_mut(4, b), Ok(&mut "b"));
     /// assert_eq!(cache.try_get_or_insert_mut(4, a), Ok(&mut "b"));
     /// ```
-    pub fn try_get_or_insert_mut<'a, F, E>(&'a mut self, k: K, f: F) -> Result<&'a mut V, E>
+    pub fn try_get_or_insert_mut<F, E>(&mut self, k: K, f: F) -> Result<&mut V, E>
     where
         F: FnOnce() -> Result<V, E>,
     {
@@ -717,6 +913,67 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    /// Returns a mutable reference to the value of the key in the cache if it is
+    /// present in the cache and moves the key to the head of the LRU list.
+    /// If the key does not exist the provided `FnOnce` is used to populate
+    /// the list and a mutable reference is returned. If `FnOnce` returns `Err`,
+    /// returns the `Err`. The value referenced by the key is only cloned
+    /// (using `to_owned()`) if it doesn't exist in the cache and `FnOnce`
+    /// succeeds.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// use std::rc::Rc;
+    ///
+    /// let key1 = Rc::new("1".to_owned());
+    /// let key2 = Rc::new("2".to_owned());
+    /// let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+    /// let f = ||->Result<String, ()> {Err(())};
+    /// let a = ||->Result<String, ()> {Ok("One".to_owned())};
+    /// let b = ||->Result<String, ()> {Ok("Two".to_owned())};
+    /// assert_eq!(cache.try_get_or_insert_mut_ref(&key1, a), Ok(&mut "One".to_owned()));
+    /// assert_eq!(cache.try_get_or_insert_mut_ref(&key2, f), Err(()));
+    /// if let Ok(v) = cache.try_get_or_insert_mut_ref(&key2, b) {
+    ///     *v = "New two".to_owned();
+    /// }
+    /// assert_eq!(cache.try_get_or_insert_mut_ref(&key2, a), Ok(&mut "New two".to_owned()));
+    /// assert_eq!(Rc::strong_count(&key1), 2);
+    /// assert_eq!(Rc::strong_count(&key2), 2); // key2 was only cloned once even though we
+    ///                                         // queried it 3 times
+    /// ```
+    pub fn try_get_or_insert_mut_ref<'a, Q, F, E>(
+        &'a mut self,
+        k: &'_ Q,
+        f: F,
+    ) -> Result<&'a mut V, E>
+    where
+        K: Borrow<Q>,
+        Q: Hash + Eq + ?Sized + alloc::borrow::ToOwned<Owned = K>,
+        F: FnOnce() -> Result<V, E>,
+    {
+        if let Some(node) = self.map.get_mut(KeyWrapper::from_ref(k)) {
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.detach(node_ptr);
+            self.attach(node_ptr);
+
+            unsafe { Ok(&mut *(*node_ptr).val.as_mut_ptr()) }
+        } else {
+            let v = f()?;
+            let (_, node) = self.replace_or_create_node(k.to_owned(), v);
+            let node_ptr: *mut LruEntry<K, V> = node.as_ptr();
+
+            self.attach(node_ptr);
+
+            let keyref = unsafe { (*node_ptr).key.as_ptr() };
+            self.map.insert(KeyRef { k: keyref }, node);
+            unsafe { Ok(&mut *(*node_ptr).val.as_mut_ptr()) }
+        }
+    }
+
     /// Returns a reference to the value corresponding to the key in the cache or `None` if it is
     /// not present in the cache. Unlike `get`, `peek` does not update the LRU list so the key's
     /// position will be unchanged.
@@ -788,7 +1045,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     ///
     /// assert_eq!(cache.peek_lru(), Some((&1, &"a")));
     /// ```
-    pub fn peek_lru<'a>(&'a self) -> Option<(&'a K, &'a V)> {
+    pub fn peek_lru(&self) -> Option<(&K, &V)> {
         if self.is_empty() {
             return None;
         }
@@ -803,6 +1060,37 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         Some((key, val))
     }
 
+    /// Returns the value corresponding to the most recently used item or `None` if the
+    /// cache is empty. Like `peek`, `peek_mru` does not update the LRU list so the item's
+    /// position will be unchanged.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(1, "a");
+    /// cache.put(2, "b");
+    ///
+    /// assert_eq!(cache.peek_mru(), Some((&2, &"b")));
+    /// ```
+    pub fn peek_mru(&self) -> Option<(&K, &V)> {
+        if self.is_empty() {
+            return None;
+        }
+
+        let (key, val);
+        unsafe {
+            let node: *mut LruEntry<K, V> = (*self.head).next;
+            key = &(*(*node).key.as_ptr()) as &K;
+            val = &(*(*node).val.as_ptr()) as &V;
+        }
+
+        Some((key, val))
+    }
+
     /// Returns a bool indicating whether the given key is in the cache. Does not update the
     /// LRU list.
     ///
@@ -934,6 +1222,34 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         unsafe { Some((key.assume_init(), val.assume_init())) }
     }
 
+    /// Removes and returns the key and value corresponding to the most recently
+    /// used item or `None` if the cache is empty.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use lru::LruCache;
+    /// use std::num::NonZeroUsize;
+    /// let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+    ///
+    /// cache.put(2, "a");
+    /// cache.put(3, "b");
+    /// cache.put(4, "c");
+    /// cache.get(&3);
+    ///
+    /// assert_eq!(cache.pop_mru(), Some((3, "b")));
+    /// assert_eq!(cache.pop_mru(), Some((4, "c")));
+    /// assert_eq!(cache.pop_mru(), None);
+    /// assert_eq!(cache.len(), 0);
+    /// ```
+    pub fn pop_mru(&mut self) -> Option<(K, V)> {
+        let node = self.remove_first()?;
+        // N.B.: Can't destructure directly because of https://github.com/rust-lang/rust/issues/28536
+        let node = *node;
+        let LruEntry { key, val, .. } = node;
+        unsafe { Some((key.assume_init(), val.assume_init())) }
+    }
+
     /// Marks the key as the most recently used one.
     ///
     /// # Example
@@ -956,7 +1272,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// cache.promote(&3);
     /// assert_eq!(cache.pop_lru(), Some((1, "a")));
     /// ```
-    pub fn promote<'a, Q>(&'a mut self, k: &Q)
+    pub fn promote<Q>(&mut self, k: &Q)
     where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
@@ -992,7 +1308,7 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
     /// assert_eq!(cache.pop_lru(), Some((1, "a")));
     /// assert_eq!(cache.pop_lru(), Some((2, "b")));
     /// ```
-    pub fn demote<'a, Q>(&'a mut self, k: &Q)
+    pub fn demote<Q>(&mut self, k: &Q)
     where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
@@ -1180,10 +1496,26 @@ impl<K: Hash + Eq, V, S: BuildHasher> LruCache<K, V, S> {
         }
     }
 
+    fn remove_first(&mut self) -> Option<Box<LruEntry<K, V>>> {
+        let next;
+        unsafe { next = (*self.head).next }
+        if !core::ptr::eq(next, self.tail) {
+            let old_key = KeyRef {
+                k: unsafe { &(*(*(*self.head).next).key.as_ptr()) },
+            };
+            let old_node = self.map.remove(&old_key).unwrap();
+            let node_ptr: *mut LruEntry<K, V> = old_node.as_ptr();
+            self.detach(node_ptr);
+            unsafe { Some(Box::from_raw(node_ptr)) }
+        } else {
+            None
+        }
+    }
+
     fn remove_last(&mut self) -> Option<Box<LruEntry<K, V>>> {
         let prev;
         unsafe { prev = (*self.tail).prev }
-        if prev != self.head {
+        if !core::ptr::eq(prev, self.head) {
             let old_key = KeyRef {
                 k: unsafe { &(*(*(*self.tail).prev).key.as_ptr()) },
             };
@@ -1466,6 +1798,7 @@ mod tests {
     use super::LruCache;
     use core::{fmt::Debug, num::NonZeroUsize};
     use scoped_threadpool::Pool;
+    use std::rc::Rc;
     use std::sync::atomic::{AtomicUsize, Ordering};
 
     fn assert_opt_eq<V: PartialEq + Debug>(opt: Option<&V>, v: V) {
@@ -1512,7 +1845,7 @@ mod tests {
     fn test_with_hasher() {
         use core::num::NonZeroUsize;
 
-        use hashbrown::hash_map::DefaultHashBuilder;
+        use hashbrown::DefaultHashBuilder;
 
         let s = DefaultHashBuilder::default();
         let mut cache = LruCache::with_hasher(NonZeroUsize::new(16).unwrap(), s);
@@ -1555,6 +1888,31 @@ mod tests {
         assert_eq!(cache.get_or_insert("lemon", || "red"), &"orange");
     }
 
+    #[test]
+    fn test_get_or_insert_ref() {
+        use alloc::borrow::ToOwned;
+        use alloc::string::String;
+
+        let key1 = Rc::new("1".to_owned());
+        let key2 = Rc::new("2".to_owned());
+        let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+        assert!(cache.is_empty());
+        assert_eq!(cache.get_or_insert_ref(&key1, || "One".to_owned()), "One");
+        assert_eq!(cache.get_or_insert_ref(&key2, || "Two".to_owned()), "Two");
+        assert_eq!(cache.len(), 2);
+        assert!(!cache.is_empty());
+        assert_eq!(
+            cache.get_or_insert_ref(&key2, || "Not two".to_owned()),
+            "Two"
+        );
+        assert_eq!(
+            cache.get_or_insert_ref(&key2, || "Again not two".to_owned()),
+            "Two"
+        );
+        assert_eq!(Rc::strong_count(&key1), 2);
+        assert_eq!(Rc::strong_count(&key2), 2);
+    }
+
     #[test]
     fn test_try_get_or_insert() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -1581,6 +1939,26 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_try_get_or_insert_ref() {
+        use alloc::borrow::ToOwned;
+        use alloc::string::String;
+
+        let key1 = Rc::new("1".to_owned());
+        let key2 = Rc::new("2".to_owned());
+        let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+        let f = || -> Result<String, ()> { Err(()) };
+        let a = || -> Result<String, ()> { Ok("One".to_owned()) };
+        let b = || -> Result<String, ()> { Ok("Two".to_owned()) };
+        assert_eq!(cache.try_get_or_insert_ref(&key1, a), Ok(&"One".to_owned()));
+        assert_eq!(cache.try_get_or_insert_ref(&key2, f), Err(()));
+        assert_eq!(cache.try_get_or_insert_ref(&key2, b), Ok(&"Two".to_owned()));
+        assert_eq!(cache.try_get_or_insert_ref(&key2, a), Ok(&"Two".to_owned()));
+        assert_eq!(cache.len(), 2);
+        assert_eq!(Rc::strong_count(&key1), 2);
+        assert_eq!(Rc::strong_count(&key2), 2);
+    }
+
     #[test]
     fn test_put_and_get_or_insert_mut() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -1602,6 +1980,22 @@ mod tests {
         assert_eq!(cache.get_or_insert_mut("lemon", || "red"), &"orange");
     }
 
+    #[test]
+    fn test_get_or_insert_mut_ref() {
+        use alloc::borrow::ToOwned;
+        use alloc::string::String;
+
+        let key1 = Rc::new("1".to_owned());
+        let key2 = Rc::new("2".to_owned());
+        let mut cache = LruCache::<Rc<String>, &'static str>::new(NonZeroUsize::new(2).unwrap());
+        assert_eq!(cache.get_or_insert_mut_ref(&key1, || "One"), &mut "One");
+        let v = cache.get_or_insert_mut_ref(&key2, || "Two");
+        *v = "New two";
+        assert_eq!(cache.get_or_insert_mut_ref(&key2, || "Two"), &mut "New two");
+        assert_eq!(Rc::strong_count(&key1), 2);
+        assert_eq!(Rc::strong_count(&key2), 2);
+    }
+
     #[test]
     fn test_try_get_or_insert_mut() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -1622,6 +2016,34 @@ mod tests {
         assert_eq!(cache.try_get_or_insert_mut(4, a), Ok(&mut "b"));
     }
 
+    #[test]
+    fn test_try_get_or_insert_mut_ref() {
+        use alloc::borrow::ToOwned;
+        use alloc::string::String;
+
+        let key1 = Rc::new("1".to_owned());
+        let key2 = Rc::new("2".to_owned());
+        let mut cache = LruCache::<Rc<String>, String>::new(NonZeroUsize::new(2).unwrap());
+        let f = || -> Result<String, ()> { Err(()) };
+        let a = || -> Result<String, ()> { Ok("One".to_owned()) };
+        let b = || -> Result<String, ()> { Ok("Two".to_owned()) };
+        assert_eq!(
+            cache.try_get_or_insert_mut_ref(&key1, a),
+            Ok(&mut "One".to_owned())
+        );
+        assert_eq!(cache.try_get_or_insert_mut_ref(&key2, f), Err(()));
+        if let Ok(v) = cache.try_get_or_insert_mut_ref(&key2, b) {
+            assert_eq!(v, &mut "Two");
+            *v = "New two".to_owned();
+        }
+        assert_eq!(
+            cache.try_get_or_insert_mut_ref(&key2, a),
+            Ok(&mut "New two".to_owned())
+        );
+        assert_eq!(Rc::strong_count(&key1), 2);
+        assert_eq!(Rc::strong_count(&key2), 2);
+    }
+
     #[test]
     fn test_put_and_get_mut() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -1745,6 +2167,23 @@ mod tests {
         assert!(cache.peek_lru().is_none());
     }
 
+    #[test]
+    fn test_peek_mru() {
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+
+        assert!(cache.peek_mru().is_none());
+
+        cache.put("apple", "red");
+        cache.put("banana", "yellow");
+        assert_opt_eq_tuple(cache.peek_mru(), ("banana", "yellow"));
+
+        cache.get(&"apple");
+        assert_opt_eq_tuple(cache.peek_mru(), ("apple", "red"));
+
+        cache.clear();
+        assert!(cache.peek_mru().is_none());
+    }
+
     #[test]
     fn test_contains() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -1830,6 +2269,41 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_pop_mru() {
+        let mut cache = LruCache::new(NonZeroUsize::new(200).unwrap());
+
+        for i in 0..75 {
+            cache.put(i, "A");
+        }
+        for i in 0..75 {
+            cache.put(i + 100, "B");
+        }
+        for i in 0..75 {
+            cache.put(i + 200, "C");
+        }
+        assert_eq!(cache.len(), 200);
+
+        for i in 0..75 {
+            assert_opt_eq(cache.get(&(74 - i + 100)), "B");
+        }
+        assert_opt_eq(cache.get(&25), "A");
+
+        assert_eq!(cache.pop_mru(), Some((25, "A")));
+        for i in 0..75 {
+            assert_eq!(cache.pop_mru(), Some((i + 100, "B")));
+        }
+        for i in 0..75 {
+            assert_eq!(cache.pop_mru(), Some((74 - i + 200, "C")));
+        }
+        for i in (26..75).into_iter().rev() {
+            assert_eq!(cache.pop_mru(), Some((i, "A")));
+        }
+        for _ in 0..50 {
+            assert_eq!(cache.pop_mru(), None);
+        }
+    }
+
     #[test]
     fn test_clear() {
         let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
@@ -2284,6 +2758,27 @@ mod tests {
         assert_eq!(cache.get_key_value("banana"), None);
     }
 
+    #[test]
+    fn test_get_key_value_mut() {
+        use alloc::string::String;
+
+        let mut cache = LruCache::new(NonZeroUsize::new(2).unwrap());
+
+        let key = String::from("apple");
+        cache.put(key, "red");
+
+        let (k, v) = cache.get_key_value_mut("apple").unwrap();
+        assert_eq!(k, &String::from("apple"));
+        assert_eq!(v, &mut "red");
+        *v = "green";
+
+        assert_eq!(
+            cache.get_key_value("apple"),
+            Some((&String::from("apple"), &"green"))
+        );
+        assert_eq!(cache.get_key_value("banana"), None);
+    }
+
     #[test]
     fn test_clone() {
         let mut cache = LruCache::new(NonZeroUsize::new(3).unwrap());
